"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var jssha_1 = __importDefault(require("jssha"));
var Client = /** @class */ (function () {
    function Client(options) {
        this.options = options;
        this.helpers = new ClientHelpers(options);
    }
    Client.prototype.get = function (endpoint) {
        var options = {};
        options.method = 'GET';
        options.headers = this.helpers.getHeaders(endpoint);
        var fullUrl = this.helpers.getBaseUrl() + endpoint;
        return fetch(fullUrl, options).then(function (result) {
            return result.ok ? result.json() : result;
        });
    };
    Client.prototype.put = function (endpoint, payload) {
        var options = {};
        options.method = 'PUT';
        options.headers = this.helpers.getHeaders(endpoint, payload);
        options.body = JSON.stringify(payload);
        var fullUrl = this.helpers.getBaseUrl() + endpoint;
        return fetch(fullUrl, options).then(function (result) {
            return result.ok ? result.json() : result;
        });
    };
    Client.prototype.post = function (endpoint, payload) {
        var options = {};
        options.method = 'POST';
        options.headers = this.helpers.getHeaders(endpoint, payload);
        options.body = JSON.stringify(payload);
        var fullUrl = this.helpers.getBaseUrl() + endpoint;
        return fetch(fullUrl, options).then(function (result) {
            return result.ok ? result.json() : result;
        });
    };
    Client.prototype.delete = function (endpoint) {
        var options = {};
        options.method = 'DELETE';
        options.headers = this.helpers.getHeaders(endpoint);
        var fullUrl = this.helpers.getBaseUrl() + endpoint;
        return fetch(fullUrl, options).then(function (result) {
            return result.ok ? result.json() : result;
        });
    };
    Client.prototype.login = function (username, password) {
        var _this = this;
        var shaObj = new jssha_1.default('SHA-1', 'TEXT');
        shaObj.update(password);
        password = shaObj.getHash('HEX');
        var endpoint = '/auth/login';
        return this.post(endpoint, { username: username, password: password }).then(function (result) {
            _this.options.authToken = result.token;
            return result;
        });
    };
    return Client;
}());
exports.Client = Client;
var ClientOptions = /** @class */ (function () {
    function ClientOptions() {
        this.apiKey = undefined;
        this.apiSecretKey = undefined;
        this.authToken = undefined;
        this.development = false;
        this.service = undefined;
        this.developmentUrl = undefined;
    }
    return ClientOptions;
}());
exports.ClientOptions = ClientOptions;
var ClientHelpers = /** @class */ (function () {
    function ClientHelpers(options) {
        this.options = options;
    }
    ClientHelpers.prototype.formatPayload = function (payload) {
        var _this = this;
        var formattedPayload = {};
        if (typeof payload === undefined || payload === null) {
            formattedPayload = null;
        }
        else if (payload !== Object(payload)) {
            formattedPayload = payload;
        }
        else if (Array.isArray(payload)) {
            formattedPayload = [];
            for (var i = 0; i < payload.length; i++) {
                formattedPayload[i] = this.formatPayload(payload[i]);
            }
        }
        else {
            Object.keys(payload).sort().forEach(function (k, v) {
                if (typeof (payload[k]) == 'object') {
                    formattedPayload[k] = _this.formatPayload(payload[k]);
                }
                else if (typeof (payload[k]) == 'string') {
                    formattedPayload[k] = payload[k].replace(/ /g, '');
                }
                else {
                    formattedPayload[k] = payload[k];
                }
            }, this);
        }
        return formattedPayload;
    };
    ClientHelpers.prototype.getBaseUrl = function () {
        if (this.options.development && this.options.developmentUrl) {
            return this.options.developmentUrl;
        }
        else if (this.options.development) {
            return 'https://api-dev.rentdynamics.com';
        }
        return 'https://api.rentdynamics.com';
    };
    ClientHelpers.prototype.getHeaders = function (endpoint, payload) {
        var headers = new Headers();
        if (this.options.apiKey && this.options.apiSecretKey) {
            if (typeof payload !== "undefined") {
                payload = this.formatPayload(payload);
            }
            var timestamp = Date.now();
            var nonce = this.getNonce(timestamp, endpoint, JSON.stringify(payload));
            if (this.options.authToken) {
                headers.append('Authorization', 'TOKEN ' + this.options.authToken);
            }
            headers.append('x-rd-api-key', this.options.apiKey);
            headers.append('x-rd-api-nonce', nonce);
            headers.append('x-rd-timestamp', timestamp.toString());
            headers.append('Content-Type', 'application/json');
            return headers;
        }
        return headers;
    };
    ClientHelpers.prototype.getNonce = function (timestamp, endpoint, payloadStr) {
        if (this.options.apiSecretKey) {
            var nonceStr = timestamp + endpoint;
            if (typeof payloadStr !== 'undefined') {
                nonceStr += payloadStr;
            }
            var shaObj = new jssha_1.default('SHA-1', 'TEXT');
            shaObj.setHMACKey(this.options.apiSecretKey, 'TEXT');
            shaObj.update(nonceStr);
            return shaObj.getHMAC('HEX');
        }
        else {
            return '';
        }
    };
    return ClientHelpers;
}());
exports.ClientHelpers = ClientHelpers;
